#ifndef FUNCTCLASS_H
#define FUNCTCLASS_H


// Abstract base class for continued fraction approximations of functions
class funct_approx
{
protected:
	REAL x, x2;		// Variables to store x and x^2

	// abstract functions to return values of a and b in expansion
	virtual REAL a(long i)=0;
	virtual REAL b(long i)=0;

	// abstract function to return coefficient of x^i in Taylor expansion
	REAL c(long i);			// gives the c(i) continued fraction approximate


public:
	void set_x(const REAL& x);
	virtual ~funct_approx() {}
	REAL approximate(long prec);
};



// Abstarct class for functions approximated using Stieltjes Continues Fraction
class stieltjes_approx : public funct_approx
{
protected:
	REAL a(long i);
	REAL b(long i);
	REAL C_Coeff(long L, long M);

public:
	REAL Pade(long L, long M);	// gives the [L/M] Pad Approximant
	REAL Stieltjes_b(long n);	// gives the bn term used within the Stieltjes type
					//  continued fraction
	REAL Stieltjes_c(long n);       // gives the bn term used within the Stieltjes type
					//  continued fraction
	REAL Taylor_a(long i);
	virtual REAL Taylor_Coeff(long i)=0;
	virtual REAL C_Base()=0;
	virtual REAL Taylor_Base()=0;
};


// Abstract class for functions approximated using Jacobi Continued Fraction
class jacobi_approx : public stieltjes_approx
{
protected:
	REAL a(long i);
	REAL b(long i);
public:
	virtual REAL Taylor_Coeff(long i)=0;
	virtual REAL C_Base()=0;
	virtual REAL Taylor_Base()=0;
};


// Derived class for Sin function
class sin_approximate : public funct_approx
{
public:
	REAL a(long i);
	REAL b(long i);
};




// Derived class for Sin function using Sieltjes Type Continued Fraction
class sin_approximate2 : public stieltjes_approx
{
public:
	REAL Taylor_Coeff(long i);
	REAL C_Base();
	REAL Taylor_Base();
};


// Derived class for Sin function using Jacobi Type Continued Fraction
class sin_approximate3 : public jacobi_approx
{
public:
	REAL Taylor_Coeff(long i);
	REAL C_Base();
	REAL Taylor_Base();
};



// Derived class for Cos function
class cos_approximate : public funct_approx
{
public:
	REAL a(long i);
	REAL b(long i);
};



// Derived class for cos function using Sieltjes Type Continued Fraction
class cos_approximate2 : public stieltjes_approx
{
public:
	REAL Taylor_Coeff(long i);
	REAL C_Base();
	REAL Taylor_Base();
};


// Derived class for cos function using Jacobi Type Continued Fraction
class cos_approximate3 : public jacobi_approx
{
public:
	REAL Taylor_Coeff(long i);
	REAL C_Base();
	REAL Taylor_Base();
};


// Derived class for Exp function
class exp_approximate : public funct_approx
{
public:
	REAL a(long i);
	REAL b(long i);
};


// Derived class for exp function using Stieltjes Type Continued Fraction
class exp_approximate2 : public stieltjes_approx
{
public:
	REAL Taylor_Coeff(long i);
	REAL C_Base();
	REAL Taylor_Base();
};


// Derived class for exp function using Jacobi Type Continued Fraction
class exp_approximate3 : public jacobi_approx
{
public:
	REAL Taylor_Coeff(long i);
	REAL C_Base();
	REAL Taylor_Base();
};


// Derived class for Tan function
class tan_approximate : public funct_approx
{
public:
	REAL a(long i);
	REAL b(long i);
};


// Derived class for arctan function
class tanh_approximate : public funct_approx
{
public:
	REAL a(long i);
	REAL b(long i);
};


// Derived class for arctan function
class atan_approximate : public funct_approx
{
public:
	REAL a(long i);
	REAL b(long i);
};


// Derived class for arctan function
class atanh_approximate : public funct_approx
{
public:
	REAL a(long i);
	REAL b(long i);
};


#endif